import sys
from pyfaidx import Fasta
from sys import argv
from typing import Dict

"""
Params: source file (in fasta) | target file (in fasta) | name of chromosome to change (chr_) | copy number of chr_  
"""

chromosome_names = ['chr1', 'chr10', 'chr11', 'chr12', 'chr13', 'chr14', 'chr15', 'chr16', 'chr17', 'chr18',
                    'chr19', 'chr2', 'chr20', 'chr21', 'chr22', 'chr3', 'chr4', 'chr5', 'chr6', 'chr7',
                    'chr8', 'chr9', 'chrX', 'chrY']

source = argv[1]
target = argv[2]

original = Fasta(source)

# Collect chromosomes ids with their respective copy number

last_parameters = iter(argv[3:])
chromosome_CN: Dict[str, str] = dict([*zip(last_parameters, last_parameters)])


print("Source: {}\nTarget: {}\n".format(source, target))
for chr_name, copy_number in chromosome_CN.items():
    if chr_name not in chromosome_names:
        print("ERROR: Wrong name of target chromosome", file=sys.stderr)
        sys.exit(1)
    if int(copy_number) < 0:
        print("ERROR: Negative number of chromosomes", file=sys.stderr)
        sys.exit(2)
    print("Affected chromosome: {} with copy number {}".format(chr_name, copy_number))


def write_sequence(gseq):
    for i in range(0, len(gseq), 80):
        f.write('{}\n'.format(gseq[i:i + 80]))


# Open target and copy every chromosome two times, except agrv[3]. Argv[3] do agrv[4] times
with open(target, 'w') as f:
    for gid in original.keys():
        gseq = original[gid]
        if gid in chromosome_CN.keys():
            for j in range(int(chromosome_CN[gid])):
                f.write('>{}.{}\n'.format(gid, j))
                write_sequence(gseq)
            print("Changed {} chromosome, now there are {} of them".format(gid, int(chromosome_CN[gid])))
            continue
        # If not a targeted sequence, put two copies of itself or one if sex chromosome
        f.write('>{}\n'.format(gid))
        write_sequence(gseq)
        if gid not in ['chrX', 'chrY']:
            f.write('>{}.second\n'.format(gid))
            write_sequence(gseq)

print("SUCCESS: Chromosomes written to {}\n".format(target))

result = Fasta(target)

for i, rec in enumerate(result):
    print("{}: {} length: {}".format(i + 1, rec.name, len(rec)))
